@@ -11,6 +11,11 @@ of the License, or (at your option) any later version.
 
 [cpscotti, roboscotti, scotti@ieee.org]
 
+Sincere thanks to No!No!No!Yes!@talk.maemo.org for adding some desired features that I "rejected" just for being lazy ....lol
+(nobody looks at the source code anyway)
+For more info:
+http://talk.maemo.org/showthread.php?t=39177
+
 */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -49,7 +54,10 @@ of the License, or (at your option) any later version.
 
 
 #define SIZE_WIDTH_ALL 800
-#define SIZE_HEIGHT_LINE 40
+#define SIZE_WIDTH_MIN_RATIO 0.118
+
+#define SIZE_HEIGHT_LINE 31
+#define SIZE_HEIGHT_RATIO_DEFAULT 1.29
 
 void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean newAll, gboolean newInstance);
 gboolean desktop_cmd_exec_update_content (DesktopCmdExec *self);
@@ -91,6 +99,7 @@ struct _DesktopCmdExecPrivate
 		gboolean updOnDesktop;
 		guint updNeworkPolicy;
 		gdouble widthRatio;
+		gdouble heightRatio;//AP
 		
 		gchar * instanceTitle;
 		gchar * instanceCmd;
@@ -191,6 +200,18 @@ void desktop_cmd_exec_read_settings ( DesktopCmdExec *self )
 				error = NULL;
 			}
 
+/* AP; No!No!No!Yes!, thanks! */
+			self->priv->heightRatio = g_key_file_get_double (keyFile, self->priv->widgetID, "heightRatio", &error);
+			if (error) {
+				//no value? assume std
+				noInstanceKeysFlag = TRUE;
+				self->priv->heightRatio = SIZE_HEIGHT_RATIO_DEFAULT;
+
+				g_warning("no heightRatio on key file, assuming SIZE_HEIGHT_RATIO_DEFAULT");
+				g_error_free (error);
+				error = NULL;
+			}
+
 			g_free(self->priv->instanceTitle);
 			self->priv->instanceTitle = g_key_file_get_string (keyFile, self->priv->widgetID, "instanceTitle", &error);
 			if (error) {
@@ -273,6 +294,7 @@ void desktop_cmd_exec_read_settings ( DesktopCmdExec *self )
 		self->priv->delaySeconds = 0;
 
 		self->priv->widthRatio = 1.0;
+		self->priv->heightRatio = SIZE_HEIGHT_RATIO_DEFAULT;
 
 		self->priv->instanceTitle = NULL;
 		self->priv->instanceCmd = NULL;
@@ -326,7 +348,7 @@ void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean newAll, gbo
 		self->priv->c_size = 4;
 		gchar * p_titles[] = { "Uptime:", "Battery(%):", "Battery(mAh):", "Boot Reason:", NULL};
 		gchar * p_commands[] = {
-		"uptime | awk 'NF == 7 {print $2 \" \" $3; exit 0}; NF == 8 {print $2 \" \" $3; exit 0}; NF == 10 {print $3 \" \" $4 \" \" $5; exit 0}; {print $3 \" \" $4 \" \" $5}'",
+		"uptime|cut -d\" \" -f3-|cut -d\",\" -f-1",
 		"hal-device | grep percen | awk '{print $3 \"%\"}'",
 		"hal-device | grep battery.reporting | awk -F. '{print $3}' | sort | awk '$1 == \"current\" { current = $3}; $1== \"design\" {print current \"/\" $3}'",
 		"cat /proc/bootreason", NULL};
@@ -355,6 +377,7 @@ void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean newAll, gbo
 		}
 		
 		self->priv->widthRatio = 1.0;
+		self->priv->heightRatio = SIZE_HEIGHT_RATIO_DEFAULT;
 		self->priv->updOnClick = TRUE;
 		self->priv->updOnDesktop = TRUE;
 		self->priv->updNeworkPolicy = NETWORK__UNRELATED;
@@ -370,6 +393,7 @@ void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean newAll, gbo
 	}
 
 	g_key_file_set_double (keyFile, self->priv->widgetID, "widthRatio", self->priv->widthRatio);
+	g_key_file_set_double (keyFile, self->priv->widgetID, "heightRatio", self->priv->heightRatio);
 
 	if(self->priv->instanceTitle != NULL && self->priv->instanceCmd != NULL)
 	{
@@ -478,6 +502,10 @@ void desktop_cmd_exec_content_create (DesktopCmdExec *self)
 
 	self->priv->cmdTitle_lb = gtk_label_new ("title:");
 	self->priv->cmdResult_lb = gtk_label_new ("result");
+
+	//gtk_label_set_line_wrap (GTK_LABEL (self->priv->cmdResult_lb), TRUE);
+	// wraps when unnecessary and doesn't wrap when necessary =/..
+	/* some insight on this at: http://library.gnome.org/devel/gtk/2.15/GtkLabel.html#gtk-label-set-line-wrap */
 	
 	GtkWidget *box = gtk_hbox_new (FALSE, 0);
 	
@@ -486,8 +514,8 @@ void desktop_cmd_exec_content_create (DesktopCmdExec *self)
 	
 	hildon_helper_set_logical_color (self->priv->cmdResult_lb, GTK_RC_FG, GTK_STATE_NORMAL, "ActiveTextColor");
 
-	gtk_misc_set_alignment (GTK_MISC (self->priv->cmdTitle_lb), 0, 1);
-	gtk_misc_set_alignment (GTK_MISC (self->priv->cmdResult_lb), 1, 1);
+	gtk_misc_set_alignment (GTK_MISC (self->priv->cmdTitle_lb), 0, 0);//Changed Alignment to Top-Left //AP
+	gtk_misc_set_alignment (GTK_MISC (self->priv->cmdResult_lb), 0, 0);//Changed Alignment to Top-Left //AP
 
 	gtk_misc_set_padding (GTK_MISC (self->priv->cmdTitle_lb), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
 	gtk_misc_set_padding (GTK_MISC (self->priv->cmdResult_lb), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
@@ -513,8 +541,13 @@ gboolean desktop_cmd_exec_resize_hack(DesktopCmdExec *self)
 		desktop_cmd_exec_read_settings (self);
 		desktop_cmd_exec_update_content (self);
 
-		gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
-		gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
+// 		gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
+// 		gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
+
+/* AP; No!No!No!Yes!, thanks! */
+		gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), (int)(SIZE_HEIGHT_LINE*self->priv->heightRatio));
+		gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), (int)(SIZE_HEIGHT_LINE*self->priv->heightRatio));
+		
 	}
 // 	else
 // 	{
@@ -542,7 +575,8 @@ gboolean desktop_cmd_exec_update_content (DesktopCmdExec *self)
 
 	
 	FILE *fp;
-	gchar line[256];
+	gchar line[2048];
+	size_t l;//AP
 	gchar *result;
 	
 	gboolean found = FALSE;
@@ -557,10 +591,13 @@ gboolean desktop_cmd_exec_update_content (DesktopCmdExec *self)
 	gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), self->priv->instanceTitle);
 
 	fp = popen (self->priv->instanceCmd, "r");
-	
-	while (fgets (line, sizeof line, fp)) {
-		result = g_strstrip (line);
-		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), result);
+
+	/* AP; No!No!No!Yes!, thanks! */
+	//	while (fgets (line, sizeof line, fp)) {//Needed to change this "line-based" management for output //AP
+	if (l=fread (line, 1, sizeof line, fp)) {//to this "buffer-based" for multiline handling in widget //AP
+		line[l-1]='\000';//AP
+
+		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), line);//AP
 		found = TRUE;
 	}
 	pclose (fp);
@@ -728,12 +765,12 @@ static void desktop_cmd_exec_init (DesktopCmdExec *self)
 	hints.min_width = (int)(SIZE_WIDTH_ALL*0.05);
 	hints.min_height = SIZE_HEIGHT_LINE;
 	hints.max_width = (int)(SIZE_WIDTH_ALL);
-	hints.max_height = SIZE_HEIGHT_LINE;
+	hints.max_height = SIZE_HEIGHT_LINE*10;
 
 	gtk_window_set_geometry_hints (GTK_WINDOW (self), self->priv->homeWidget, &hints, GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
 
-	gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*0.118), SIZE_HEIGHT_LINE);
-	gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*0.118), SIZE_HEIGHT_LINE);
+	gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*SIZE_WIDTH_MIN_RATIO), SIZE_HEIGHT_LINE);
+	gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*SIZE_WIDTH_MIN_RATIO), SIZE_HEIGHT_LINE);
 
 	g_timeout_add (500, (GSourceFunc)desktop_cmd_exec_resize_hack, self);
 
@@ -810,7 +847,8 @@ void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
 
 		GtkWidget *widthBox = gtk_hbox_new (FALSE, 0);
 
-		GtkWidget *widthLabel = gtk_label_new ("Width:");
+		GtkWidget *widthLabel = gtk_label_new ("");
+		gtk_label_set_markup (GTK_LABEL (widthLabel), "Width:\n<small>(relative to screen)</small>");
 		gtk_misc_set_alignment (GTK_MISC (widthLabel), 0, 0.5);
 		gtk_misc_set_padding (GTK_MISC (widthLabel), HILDON_MARGIN_DOUBLE, 0);
 		gtk_size_group_add_widget (group, widthLabel);
@@ -822,6 +860,29 @@ void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
 		gtk_box_pack_start (GTK_BOX (widthBox), widthSelect, TRUE, TRUE, 0);
 
 		gtk_container_add (GTK_CONTAINER (content_area), widthBox);
+
+		//layout opts heightBox //AP
+		GtkWidget *heightBox = gtk_hbox_new (FALSE, 0);//AP
+		
+		GtkWidget *heightLabel = gtk_label_new ("");//AP
+		
+		gtk_label_set_markup (GTK_LABEL (heightLabel), "Height:\n<small>(number of lines)</small>");
+		
+		gtk_misc_set_alignment (GTK_MISC (heightLabel), 0, 0.5);//AP
+		gtk_misc_set_padding (GTK_MISC (heightLabel), HILDON_MARGIN_DOUBLE, 0);//AP
+		gtk_size_group_add_widget (group, heightLabel);//AP
+		
+		GtkWidget * heightSelect = gtk_hscale_new_with_range ((gdouble)1.0, (gdouble)10.0, (gdouble)0.1);//AP
+		
+
+	
+		gtk_range_set_value(GTK_RANGE(heightSelect), self->priv->heightRatio);//AP
+		
+		gtk_box_pack_start (GTK_BOX (heightBox), heightLabel, FALSE, FALSE, 0);//AP
+		gtk_box_pack_start (GTK_BOX (heightBox), heightSelect, TRUE, TRUE, 0);//AP
+		
+		gtk_container_add (GTK_CONTAINER (content_area), heightBox);//AP
+		
 		//END
 
 		//BEGIN Update Policy
@@ -898,9 +959,12 @@ void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
 			}
 
 			self->priv->widthRatio = gtk_range_get_value(GTK_RANGE(widthSelect));
+			self->priv->heightRatio = gtk_range_get_value(GTK_RANGE(heightSelect));
 
-			gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
-			gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
+// 			gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
+// 			gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
+			gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), (int)(SIZE_HEIGHT_LINE*self->priv->heightRatio));//AP
+			gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), (int)(SIZE_HEIGHT_LINE*self->priv->heightRatio));//AP
 			
 			self->priv->updOnClick = hildon_check_button_get_active (HILDON_CHECK_BUTTON (checkBtClick));
 			self->priv->updOnDesktop = hildon_check_button_get_active (HILDON_CHECK_BUTTON (checkBtDesktop));
